/*
 * Copyright (C) 2010 Novell, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 * Author: Wei Kong <wkong@novell.com>
 * Contributor: Jim Fehlig <jfehlig@novell.com>
 *              Chunyan Liu <cyliu@novell.com>
 */

#include <xenctrl.h>
#include <xs.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "xen-utils.h"


char *get_vncport(struct xs_handle *xsh, uint32_t domid)
{
	unsigned int len;
	char *path;
	char *s;

    if (asprintf(&path, "/local/domain/%u/console/vnc-port", domid) < 0)
        return NULL;
	
	s = xs_read(xsh, XBT_NULL, path, &len);
    free(path);
	
	return s;
}

char *domid_to_domname(struct xs_handle *xsh, uint32_t domid)
{
	unsigned int len;
	char *path;
	char *s;

    if (asprintf(&path, "/local/domain/%u/vm", domid) < 0)
        return NULL;

    s = xs_read(xsh, XBT_NULL, path, &len);
    free(path);

    if (asprintf(&path, "%s/name", s) < 0) {
        free(s);
        return NULL;
    }

    free(s);
    s = xs_read(xsh, XBT_NULL, path, &len);
    free(path);

    return s;
}

char *domid_to_vm_uuid(struct xs_handle *xsh, uint32_t domid)
{
	unsigned int len;
	char *path;
	char *s;

    if (asprintf(&path, "/local/domain/%u/vm", domid) < 0)
        return NULL;

	s = xs_read(xsh, XBT_NULL, path, &len);
    free(path);

    return s;
}

void print_dom(xc_dominfo_t dominfo,
               const char *name,
               const char *vncport,
               int compact)
{
    unsigned long dommem = (dominfo.nr_pages * (XC_PAGE_SIZE / 1024UL)) / 1024UL;

    if (compact) {
        printf("%-40s %5d %5lu %5d      %c%c%c%c%c  %8.1f  %s\n",
               name,
               dominfo.domid,
               dommem,
               dominfo.nr_online_vcpus,
               dominfo.running ? 'r' : '-',
               dominfo.blocked ? 'b' : '-',
               dominfo.paused ? 'p' : '-',
               dominfo.dying ? 'd' : '-',
               dominfo.crashed ? 'c' : '-',
               ((float)dominfo.cpu_time / 1e9),
               vncport ? vncport : "Noport");
    } else {
        printf("Name:	%s\n", name);
        printf("ID:	%d\n", dominfo.domid);
        printf("Mem:	%lu\n", dommem);
        printf("VCPUs:	%d\n", dominfo.nr_online_vcpus); 
        printf("State:	%c%c%c%c%c\n",
               dominfo.running ? 'r' : '-',
               dominfo.blocked ? 'b' : '-',
               dominfo.paused ? 'p' : '-',
               dominfo.dying ? 'd' : '-',
               dominfo.crashed ? 'c' : '-');
        printf("Time:	%8.1f\n", ((float)dominfo.cpu_time / 1e9));
        printf("VPort:	%s\n", vncport ? vncport : "Noport");
    }
}

int xlist(int argc, char **argv)
{
	int rcode = 1;
	xc_dominfo_t *info = NULL;
	uint32_t first_dom = 0;
	int max_doms = 1024, nr_doms, i;
	int xc_handle = 0;
	struct xs_handle *xsh = NULL;
	char *domname, *vncport;

    if (argc > 2) {
		fprintf(stderr, "usage: %s [domU-name]\n", argv[0]);
		return 1;
    }
        
	xc_handle = xc_interface_open();
	if (xc_handle < 0) {
		perror("Unable to open libxc handle.");
		goto out;
	}

	xsh = xs_daemon_open();
	if (!xsh) {
		perror("Unable to open libxenstore handle.");
		goto out;
	}

	info = calloc(max_doms, sizeof(xc_dominfo_t));
	if (info == NULL) {
		perror("Unable to allocate memory.");
		goto out;
	}

	nr_doms = xc_domain_getinfo(xc_handle, first_dom, max_doms, info);
	if (nr_doms < 0) {
		perror("xc_domain_getinfo() failed.");
		goto out;
	}

	if (argc == 1) {
		printf("Name                                        ID   Mem VCPUs\tState\tTime(s)\tVnc-port\n");
		for (i = 0; i < nr_doms; i++) {
			domname = domid_to_domname(xsh, info[i].domid);
			vncport = get_vncport(xsh, info[i].domid);
            print_dom(info[i], domname, vncport, 1);
			free(domname);
			free(vncport);
		}
        rcode = 0;
	} else if (argc == 2) {
		for (i = 0; i < nr_doms; i++) {
			domname = domid_to_domname(xsh, info[i].domid);
			if (domname && strncmp(argv[1], domname, strlen(argv[1])+1) == 0) {
				vncport = get_vncport(xsh, info[i].domid);
                print_dom(info[i], domname, vncport, 0);
				free(domname);
				free(vncport);
				rcode = 0;
				goto out;
			}
            free(domname);
		}
		if ( i == nr_doms ) {
			fprintf(stderr, "Unable to find domain named %s\n", argv[1]);
			goto out;
		}
	}
    
out:
	if (xsh)
		xs_daemon_close(xsh);
	if (info)
		free(info);
	if (xc_handle)
		xc_interface_close(xc_handle);
	return rcode;
}
